
#include "utils.hpp"
#include <LEDA/map.h>
#include <LEDA/templates/shortest_path.t>

extern char *devnames[];

// This function will instantiate nodes in the Task Graph to specific device addresses
template <class VT, class ET, class VG, class EG>
void Embed(GRAPH<VT,ET>& TG, GRAPH<VG,EG>& G, const node& tgnode, const node& gnode, 
					 GraphWin& gw, GraphWin& gwtask) {

	// starting vertex: tgnode; mark and instantiate tgnode
	TG[tgnode].is_visited() = TRUE;
	TG[tgnode].inst() = G[gnode].get_addr();

	// Mark gnode as visited and instantiated
	G[gnode].is_visited() = TRUE;
	//	G[gnode].is_instantiated() = TRUE;
	gw.set_color(gnode,blue);

	queue<node> TGQ;
	queue<node> GQ;

	GQ.append(gnode);
	TGQ.append(tgnode);


	// BFS iterate through the task graph
	while (TGQ.empty() == FALSE)
	{
		node v = TGQ.pop();
		node w = GQ.pop();

		TG[v].is_visited() = TRUE;

		node p;

		forall_adj_nodes(p,v) {
			VT & tp = TG[p];
			if (tp.is_visited() == TRUE) continue;

			node nearest;
			FindNearest(G, w, tp.get_dtype(), nearest);

			tp.is_visited() = TRUE;
				
			tp.inst() = G[nearest].get_addr();
			gw.set_color(nearest,red);
				
			TGQ.append(p);
			GQ.append(nearest);
		}
	}

}





//Find a node of type "devtype" nearest to "snode"
void FindNearest(GRAPH<SpecNode,int>& G, const node& gnode, DeviceType devtype, node& nearest) {

	// unvisit all nodes
	node w;
	forall_nodes(w,G) G[w].is_visited() = FALSE;


	BFS_It it(G,gnode);

// 	cout << "Source of search: "; G[gnode].PrintNode(); 
// 	cout << "\tDevice Type to search: " << print_devtype(devtype) << endl;

	while (it.valid()) {
		w = it.get_node();
		SpecNode& ss = G[w];

		if (ss.is_visited() || ss.is_instantiated()) { ++it; continue; }
// 		cout << '\t'; ss.PrintNode();

		if (ss.get_dtype() == devtype) {
				ss.is_instantiated() = TRUE;
				nearest = w;
				break;
		}
		
		ss.is_visited() = TRUE;
		
		++it;
	}

	if (G[w].is_instantiated() == FALSE) {
		cout << "Device of this type not found. Exiting..." << endl; 
		exit(0);
	}
 
	return;
}



// This function will instantiate nodes in the Task Graph to specific device addresses
// Create an MST of the task graph and then embed the MST into the network
template <class VT, class ET, class VG, class EG>
void EmbedMaxDeg(GRAPH<VT,ET>& TG, GRAPH<VG,EG>& G, const node& tgnode, const node& gnode,
								 GraphWin& gw, GraphWin& gwtask) {

	// starting vertex: tgnode; mark and instantiate tgnode
	TG[tgnode].is_visited() = TRUE;
	TG[tgnode].inst() = G[gnode].get_addr();

	G[gnode].is_visited() = TRUE;
	//	G[gnode].is_instantiated() = TRUE;

	//	gw.set_color(gnode,blue);

	int degree = 0, d;
	node nmax, v;

	// Find the node with the maximum degree in TG
	forall_nodes(v,TG) {
		d = TG.degree(v);
		if (d > degree)
		{  degree = d; nmax = v;}
	}

	cout << "Node with maximum degree in TG: "; G[nmax].PrintNode(); 
	cout << "\tDegree = " << degree << endl; 

	// Now start the BFS from nmax in TG
	node nearest;
	FindNearest(G, gnode, TG[nmax].get_dtype(), nearest);	

	Embed(TG,G,nmax,nearest,gw,gwtask);

}



void TaskEmbed(GRAPH<SpecNode,int>& G, GRAPH<TaskGNode,int>& TG, node& w, GraphWin& gw, GraphWin& gwtask)
{
	bool b;

	b = Is_Connected(TG);
	if (b) cout << "TG is connected" << endl;
	else {cout << "TG is partitioned" << endl; exit(0);}

	b = Is_Connected(G);
	if (b) cout << "G is connected" << endl;
	else cout << "G is partitioned" << endl;


	// Find user node in the Task Graph
	node v; b = FALSE;
	forall_nodes(v,TG) {
		if (TG[v].get_dtype() == USER_PDA) 
		{ b = TRUE; break; } 
	}
	if (b) {cout << "User TG node = "; TG[v].PrintNode(); cout << endl;}
	else { cout << "No User PDA found in Task Graph" << endl; exit(0); }

	Embed(TG,G,v,w,gw,gwtask);

	node p;
	forall_nodes(p,TG) { TG[p].PrintNode(); cout << endl; }

	cout << endl;
}



void TaskEmbedMaxDeg(GRAPH<SpecNode,int>& G, GRAPH<TaskGNode,int>& TG, node& w, GraphWin& gw, GraphWin& gwtask)
{
	bool b;

	b = Is_Connected(TG);
	if (b) cout << "TG is connected" << endl;
	else {cout << "TG is partitioned" << endl; exit(0);}

	b = Is_Connected(G);
	if (b) cout << "G is connected" << endl;
	else cout << "G is partitioned" << endl;


	// Find user node in the Task Graph
	node v; b = FALSE;
	forall_nodes(v,TG) {
		if (TG[v].get_dtype() == USER_PDA) 
		{ b = TRUE; break; } 
	}
	if (b) {cout << "User TG node = "; TG[v].PrintNode(); cout << endl;}
	else { cout << "No User PDA found in Task Graph" << endl; exit(0); }

	EmbedMaxDeg(TG,G,v,w,gw,gwtask);

	node p;
	forall_nodes(p,TG) { TG[p].PrintNode(); cout << endl; }

	cout << endl;
}




int ComputeDilation(GRAPH<SpecNode,int>& G, GRAPH<TaskGNode,int>& TG)
{
	int path_length = 0, dilation = 0;
	map<int,node> M;

	node m;
	forall_nodes(m,G) M[G[m].get_addr()] = m;
	
	node v;
	forall_nodes(v,TG) {
		node w;
		forall_adj_nodes(w,v) {

			node_array<edge> pred(G); 

			path_length = DIJKSTRA(G, M[TG[v].inst()], M[TG[w].inst()], G.edge_data(), pred);
			//			cout << "Path length = " << path_length << "\tDilation = " << dilation << endl;
			// (path_length > dilation) dilation = path_length;
			dilation += path_length;
		}
	}
	
	return dilation;
}

// Computes n!/r!
inline unsigned long f(int n, int r)
{
	if (n<r) {cout << "Not enough devices" << endl; exit(0);}

	unsigned long res = 1;
	for (int i=n; i>n-r; i--)
		res *= i;

	return res;
}


#define MAXTYPE 20 // max devices in the network of a type

int ComputeOptimalDilation(GRAPH<SpecNode,int>& G, GRAPH<TaskGNode,int>& TG)
{
	node devcatg [DEVNUM][MAXTYPE];
	int count[DEVNUM], i=0, j=0;

	// initialize
	for (i=0; i<DEVNUM; i++) count[i] = 0;

	node v;
	forall_nodes(v,G) {
		if ((j = (int) G[v].get_dtype()) == USER_PDA) continue;
		
		devcatg [j][count[j]] = v;
		count[j] ++;
	}
	
	cout << "Devices in Categories" << endl;
	for (i=0; i<DEVNUM; i++) {
		for (j=0; j<count[i]; j++)
		{ G[devcatg[i][j]].PrintNode(); cout << '\t'; }
		cout << endl;
	}

	// Calculate total number of embeddings possible
	int numocc[DEVNUM];
	for (i=0; i<DEVNUM; i++) numocc[i] = 0;

	node w;
	forall_nodes(w,TG) {
		if ((j = (int) TG[w].get_dtype()) == USER_PDA) continue;
		numocc[j] ++;
	}

	unsigned long numembd = 1, ff = 1;
	for (i=0; i<DEVNUM; i++) {
		if (numocc[i]==0) continue;
		else if (numocc[i]==1) ff = count[i];
		else ff = f(count[i],numocc[i]);

		numembd *= ff;
	}
	cout << "# Embeddings possible = " << numembd << endl;

	// Compute dilation of each embedding
	/*
	node v;
	forall_nodes(v,TG) {
		node w;
		forall_adj_nodes(w,v) {

			node_array<edge> pred(G); 

			path_length = DIJKSTRA(G, M[TG[v].inst()], M[TG[w].inst()], G.edge_data(), pred);
			//			cout << "Path length = " << path_length << "\tDilation = " << dilation << endl;
			if (path_length > dilation) dilation = path_length;
		}
		}*/

	
	return 0;

}
