
#include <unistd.h> // for sleep()
#include <ctime>
#include "utils.hpp"

void main(int argc, char **argv)
{

  int maxdegdl = 0, userdl = 0;

  if (argc < 7) {
    cout << "Usage: "<< argv[0] << " <d> |V| |R| <d2> |V_TG| |R_TG|\n";
    exit(1);
  }
  
  time_t t;
  srandom(time(&t));

  /***** Generate random topology *****/
  int d = atoi(argv[1]);
  int n = atoi(argv[2]);
  double R = atof(argv[3]);

  GRAPH<SpecNode,int> G;

  mypoint mp[n+1];
  node vn[n+1];

  gengraph(G,d,n,R,&mp[0],&vn[0]);

  node_array<point> pos(G);
  assign_coord(G,pos,&mp[0],&vn[0]);


  /***** Generate random task graph *****/
  int d2 = atoi(argv[4]);
  int n2 = atoi(argv[5]);
  double R2 = atof(argv[6]);

  GRAPH<TaskGNode,int> TG;
  gentaskgraph(TG,d2,n2,R2);


  GraphWin gw(G,"Physical Topology Graph");

  GraphWin gwtask(TG,"Task Graph");
  /* 
     gwtask.set_node_width(30);
     gwtask.set_node_height(30);
     gwtask.set_node_label_type(data_label);
     gwtask.set_node_label_font(roman_font,10);
     gwtask.set_edge_direction(undirected_edge);
     gwtask.set_show_status(false);

     gwtask.place_into_win();
     gwtask.display();
     gwtask.edit();*/


  /***** Insert the User Node into G *****/

#define NN 50

  for (int l=0; l<2*NN; l++)
  {
    double xpos = uniform() * d;
    double ypos = uniform() * d;

    if (l%2) {
      SpecNode start(n+1,USER_PDA,n+1);
      vn[n] = G.new_node(start);
      point p(xpos,ypos);

      // Add Edges
      for (int i=0; i<n; i++) {
	if (p.sqr_dist(pos[vn[i]]) < R*R)
	  G.new_edge(vn[n],vn[i],1);
      }

      // Update the position array
      node_array<point> pos1(G);
      for (int i=0; i<n; i++)
	pos1[vn[i]] = pos[vn[i]];

      pos1[vn[n]] = p;

      /***** Call Embedding Routines *****/
      TaskEmbed(G,TG,vn[n],gw,gwtask);
      int dl = ComputeDilation(G,TG);
      userdl += dl;
      cout << "Dilation (user node) = " << dl << endl;

      G.del_node(vn[n]);
      node v;
      forall_nodes(v,TG) {
	TaskGNode& tt = TG[v];
	tt.inst() = -1;
	tt.is_visited() = FALSE;
      }

      forall_nodes(v,G) {
	SpecNode& ss = G[v];
	ss.is_visited() = FALSE;
	ss.is_instantiated() = FALSE;
      }
    }
    else {
      SpecNode start(n+1,USER_PDA,n+1);
      vn[n] = G.new_node(start);
      point p(xpos,ypos);

      // Add Edges
      for (int i=0; i<n; i++) {
	if (p.sqr_dist(pos[vn[i]]) < R*R)
	  G.new_edge(vn[n],vn[i],1);
      }

      // Update the position array
      node_array<point> pos1(G);
      for (int i=0; i<n; i++)
	pos1[vn[i]] = pos[vn[i]];

      pos1[vn[n]] = p;

      /***** Call Embedding Routines *****/
      TaskEmbedMaxDeg(G,TG,vn[n],gw,gwtask);
      int dl = ComputeDilation(G,TG);
      maxdegdl += dl;
      cout << "Dilation (max deg) = " << dl << endl;

      G.del_node(vn[n]);
      node v;
      forall_nodes(v,TG) {
	TaskGNode& tt = TG[v];
	tt.inst() = -1;
	tt.is_visited() = FALSE;
      }

      forall_nodes(v,G) {
	SpecNode& ss = G[v];
	ss.is_visited() = FALSE;
	ss.is_instantiated() = FALSE;
      }
    }
	

    sleep(1);

    //	int opt = ComputeOptimalDilation(G,TG);

    /***** Viaualization *****/
    /*		gw.set_node_width(30);
		gw.set_node_height(30);
		gw.set_position(pos1);
		gw.set_node_label_type(data_label);
		gw.set_node_label_font(roman_font,10);
		gw.set_edge_direction(undirected_edge);
		gw.set_show_status(false);

		gw.place_into_win();
		gw.display();
		gw.edit(); */
  }

  cout << "Average dilation: User Node case = " << userdl/NN << "\t Max Degree Case = " << maxdegdl/NN << endl;
}
