#ifndef __NODEHPP__
#define __NODEHPP__

// This file enlists the structure of nodes in the ad hoc network 
// as well as those in a task graph

#include <iostream>

#include "device.hpp"

typedef int ID;
typedef int Address;
enum boolean { TRUE = 1, FALSE = 0};

class SpecNode {
	ID          node_id;
	DeviceType  d_type;
	Address     node_addr;
	boolean     visited;
	boolean     instantiated;
	//	int         lvl; // wrt the search root

 public:	
	SpecNode() { };

	SpecNode (ID id, DeviceType dt, Address addr) : 
		node_id (id), 
		d_type (dt),
		node_addr (addr),
		visited (FALSE), 
		instantiated (FALSE) { };
	//		lvl (0) { };

	SpecNode (ID id, Address addr) : 
		node_id (id), 
		d_type (UNKNOWN),
		node_addr (addr),
		visited (FALSE),
		instantiated (FALSE) { };
		//		lvl (0) { };

	ID&         get_id() { return node_id; }
	Address&    get_addr() { return node_addr; }
	DeviceType& get_dtype() { return d_type; }
	boolean&    is_visited() { return visited; }
	boolean&    is_instantiated() { return instantiated; }
	//	int&        level() {return lvl;}

	void PrintNode() {
		cout << '{' << get_addr() << ',' << print_devtype(get_dtype()) << '}';
	}

	friend istream& operator>> (istream& ifstr, SpecNode& s) {
		int id;
		char devtype[20];

		ifstr >> id;
		ifstr >> devtype;

		s = SpecNode(id, sttodev(devtype), id);
		return ifstr;
	};

 	friend ostream& operator<< (ostream& ofstr, SpecNode s) {
		ofstr << s.get_addr() << ' ' <<  print_devtype(s.get_dtype());
		return ofstr;
	}
};


class TaskGNode {
	ID         node_id;
	DeviceType d_type;
	int        instance;
	boolean    visited;

public:
	TaskGNode () {};

	TaskGNode (ID id, DeviceType dt) : 
		node_id (id), d_type (dt), instance (-1), visited(FALSE) { };

	TaskGNode (ID id) : 
		node_id (id), d_type (UNKNOWN), instance (-1), visited(FALSE) { };

	DeviceType& get_dtype() { return d_type; }
	ID&         get_id() { return node_id; }
	int&        inst() { return instance; }
	boolean&    is_visited() { return visited; }

	void PrintNode() {
		cout << "{" << inst() << "," << print_devtype(get_dtype()) << "," << get_id() << "," << (int)is_visited() << "}";
	}

	friend istream& operator>> (istream& ifstr, TaskGNode& t) {
		int id;
		char devtype[20];

		ifstr >> id;
		ifstr >> devtype;

		t = TaskGNode(id, sttodev(devtype));
		return ifstr;
	};

 	friend ostream& operator<< (ostream& ofstr, TaskGNode t) {
		ofstr << t.get_id() << ' ' <<  print_devtype(t.get_dtype());
		return ofstr;
	}
};


#endif
