#ifndef __UTILSHPP__
#define __UTILSHPP__

#include <LEDA/ugraph.h>
#include <LEDA/graphwin.h>
#include <LEDA/point.h>
#include <LEDA/graph_iterator.h>
#include <LEDA/basic_graph_alg.h>
#include <LEDA/graph_alg.h>
#include <LEDA/graph_misc.h>
#include <LEDA/node_array.h>
#include <LEDA/queue.h>
#include <LEDA/list.h>

#include <cstdlib>
#include <cmath>

#include "nodes.hpp"

/* Random Number Generation */
#define INVERSE_M  ((double)4.656612875e-10)
#define uniform() ((random()) * INVERSE_M)


class mypoint {
public:
  double x;
  double y;
  int id;
  char *name;

  mypoint () {};

  mypoint(double xi, double yi)
  { x = xi; y = yi; }
  
  double dist(mypoint p) {
    double xd = x - p.x, yd = y - p.y;
    return sqrt(xd*xd + yd*yd);
  }

  void print() {
    cout << id << '\t' << name << '\t' <<  '{' << x << ',' << y << '}' << endl;
  }
};


void gengraph(GRAPH<SpecNode,int>&, int, int, double, mypoint *, node *);
void assign_coord(GRAPH<SpecNode,int>&, node_array<point>&, mypoint*, node *);

void gentaskgraph(GRAPH<TaskGNode,int>&, int, int, double);


template <class VT, class ET, class VG, class EG>
void Embed(GRAPH<VT,ET>&, GRAPH<VG,EG>&, const node&, const node&, GraphWin&, GraphWin&);

void TaskEmbed(GRAPH<SpecNode,int>&, GRAPH<TaskGNode,int>&, node&, GraphWin&, GraphWin&);
void TaskEmbedMaxDeg(GRAPH<SpecNode,int>&, GRAPH<TaskGNode,int>&, node&, GraphWin&, GraphWin&);

void FindNearest(GRAPH<SpecNode,int>&, const node&, DeviceType, node&);

int ComputeDilation(GRAPH<SpecNode,int>&, GRAPH<TaskGNode,int>&);
int ComputeOptimalDilation(GRAPH<SpecNode,int>&, GRAPH<TaskGNode,int>&);

#endif


// #define MAXLINE 256

// template <class T>
// class ReadGraph {

// public:
// 	ReadGraph () {};

// 	ReadGraph (char *s, GRAPH<T,int>& G) {
// 		int num_nodes;
// 		char junk, *buf;
// 		char idstr[5]; 	bzero(idstr, 5);

// 		// Read Graph from the file 
// 		ifstream grfile;
// 		grfile.open(s, ios::in);
// 		if (!grfile) { cout << "Cannot open input file " << s << endl; exit(0); }
	
// 		grfile >> num_nodes;
// 		buf = (char *) malloc (num_nodes * MAXLINE);

// 		node vn[num_nodes];

// 		for (int i=0; i<num_nodes; ++i) { 
// 			T ss;
// 			grfile >> ss;

// 			node v = G.new_node(ss);
// 			vn[i] = v;

// 			grfile >> junk; // '-' char
// 			grfile.get(buf + i*MAXLINE, MAXLINE);	// get the remaining line in a buffer
// 		}


// 		// Now extract nodes from buf and add the edges
// 		for (int i=0; i<num_nodes; ++i) {
// 			//	forall_nodes(v,G) { // LEDA iteration macro
// 			ID id = -1;
// 			int j = 0;

// 			node v = vn[i];
// 			for (char *p = buf + i*MAXLINE; *p != '\0'; p++)	{
// 				if (isspace(*p)) continue;
// 				idstr[j++] = *p;
			
// 				if (isspace(*(p+1)) || *(p+1) == '\0') {
// 					idstr[j] = '\0';
// 					id = atoi(idstr);

// 					node w = vn[id-1];
// 					G.new_edge(v,w,1);

// 					j = 0; bzero(idstr, 5);
// 				}
// 			}
// 		}

// 		grfile.close();
// 	}
// }; 

